package com.zipcodeconway;

import java.net.Inet4Address;
import java.util.Arrays;

public class ConwayGameOfLife {
    SimpleWindow simpleWindow;
    int[][] currentGeneration;
    int[][] nextGeneration;

    public ConwayGameOfLife(Integer dimension) {
        simpleWindow = new SimpleWindow(dimension);
        nextGeneration = new int[dimension][dimension];
        currentGeneration = createRandomStart(dimension);
    }

    public ConwayGameOfLife(Integer dimension, int[][] startmatrix) {
        simpleWindow = new SimpleWindow(dimension);
        nextGeneration = new int[dimension][dimension];
        currentGeneration = createRandomStart(dimension);
        currentGeneration = startmatrix;
    }

    public static void main(String[] args) {
        ConwayGameOfLife sim = new ConwayGameOfLife(50);
        int[][] endingWorld = sim.simulate(50);
    }

    // Contains the logic for the starting scenario.
    // Which cells are alive or dead in generation 0.
    // allocates and returns the starting matrix of size 'dimension'
    private int[][] createRandomStart(Integer dimension) {
        int[][] random = new int[dimension][dimension];
        for (int i = 0; i < random.length; i++) {
            for (int j = 0; j < random.length; j++) {
                random[i][j] = (int) Math.round(Math.random());
            }
        }
        return random;
    }

    public int[][] simulate(Integer maxGenerations) {
        for (int i = 1; i <= maxGenerations; i++) {
            simpleWindow.display(currentGeneration, i);
            for (int j = 0; j < currentGeneration.length; j++) {
                for (int k = 0; k < currentGeneration.length; k++) {
                    nextGeneration[j][k] = isAlive(j,k,currentGeneration);
                }
            }
            copyAndZeroOut(nextGeneration,currentGeneration);
            simpleWindow.sleep(500);
        }
        System.out.println(Arrays.toString(currentGeneration));
        return currentGeneration;
    }

    // copy the values of 'next' matrix to 'current' matrix,
    // and then zero out the contents of 'next' matrix
    public void copyAndZeroOut(int[][] next, int[][] current) {
        for (int i = 0; i < current.length; i++) {
            for (int j = 0; j < current.length; j++) {
                current[i][j] = next[i][j];
                next[i][j] = 0;
            }
        }
    }

    // Calculate if an individual cell should be alive in the next generation.
    // Based on the game logic:
	/*
		Any live cell with fewer than two live neighbours dies, as if by needs caused by underpopulation.
		Any live cell with more than three live neighbours dies, as if by overcrowding.
		Any live cell with two or three live neighbours lives, unchanged, to the next generation.
		Any dead cell with exactly three live neighbours cells will come to life.
	*/
    private int isAlive(int row, int col, int[][] world) {
        int numberOfNeighbors = countOfNeighborCells(row, col, world);
        boolean amAlive = amIAliveRightNow(row, col, world);
        if (numberOfNeighbors == 3 && !amAlive) {
            return 1;
        } else if ((numberOfNeighbors == 2) || (numberOfNeighbors == 3) && amAlive) {
            return 1;
        }
        return 0;
    }

    private int countOfNeighborCells(int row, int col, int[][] world) {
        //System.out.println("count neighbor");
        int countOfLiveCells = 0;
        for (int i = -1; i < 2; i++) {
            //System.out.println("hi");
            for (int j = -1; j < 2; j++) {
                //System.out.println("world");
                if ((row+i < world.length) && (row+i >= 0) && (col+j < world.length) && (col+j >= 0)) {
                    if((world[row + i][col + j] == 1)) {
                        countOfLiveCells++;
                    }
                }

            }
        }
        if (amIAliveRightNow(row, col, world)) {
            countOfLiveCells--;
        }
        return countOfLiveCells;
    }

    private boolean amIAliveRightNow(int row, int col, int[][] world) {
        //System.out.println("outside" + world[row][col] + row + col);
        if (world[row][col] == 1) {
            //System.out.println("conditional" + world[row][col]);
            return true;
        }
        return false;
    }
}
